//-----------------------------------------------------------------------------
// File: mp_wrap.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "mp_wrap.h"
#ifndef SHADER_CSO
#include <D3Dcompiler.h>
#pragma comment(lib, "D3dcompiler.lib")
#endif
#include "stb_image.h"
#include "DDSTextureLoader.h"

#pragma comment(lib, "D3D11.lib")

#pragma warning ( disable : 4996)

// -------------

// eng: Class storing the textural atlas
// rus: Класс, который хранит текстурный атлас
MP_Atlas_WRAP::MP_Atlas_WRAP(int width, int height, const char* file) : MP_Atlas(width, height, file)
{
	HRESULT hr;
	ID3D11Device* device_3d=device->GetDevice3D();
	ID3D11DeviceContext* context_3d=device->GetContext3D();

	if (file_name.empty())
	{
		if (width>0 && height>0)
		{
			D3D11_TEXTURE2D_DESC desc;
			ZeroMemory(&desc, sizeof(desc));
			desc.Width=atlas_width;
			desc.Height=atlas_height;
			desc.MipLevels=1;
			desc.ArraySize=1;
			desc.Format=DXGI_FORMAT_R8G8B8A8_UNORM;
			desc.SampleDesc.Count=1;
			desc.SampleDesc.Quality=0;
			desc.Usage=D3D11_USAGE_DEFAULT;
			desc.BindFlags=D3D11_BIND_SHADER_RESOURCE;
			desc.CPUAccessFlags=0;
			desc.MiscFlags=0;

			int* zero=new int[atlas_width*atlas_height];
			ZeroMemory(zero, atlas_width*atlas_height*4);

			D3D11_SUBRESOURCE_DATA fill_zero;
			fill_zero.pSysMem=zero;
			fill_zero.SysMemPitch=atlas_width*4;
			fill_zero.SysMemSlicePitch=0;
			hr=device_3d->CreateTexture2D(&desc, &fill_zero, &texture);
			assert(hr==S_OK);
			delete []zero;
			zero=NULL;
		}
	}
	else
	{
		std::string texture_file=MP->GetPathToTexture();
		texture_file+=file_name.c_str();
		device->LoadTextureFromFile(texture_file.c_str(), false, &texture);
	}
	hr=device_3d->CreateShaderResourceView(texture, NULL, &resource_view);
	assert(hr==S_OK);
}

// eng: Destroy atlas texture
// rus: Уничтожить текстуру атласа
void MP_Atlas_WRAP::Destroy()
{
	if (resource_view)
	{
		resource_view->Release();
		resource_view=NULL;
	}

	if (texture)
	{
		texture->Release();
		texture=NULL;
	}
}

// eng: Loading of frame texture
// rus: Загрузка текстурного кадра
void MP_Atlas_WRAP::LoadTexture(const MAGIC_CHANGE_ATLAS* c)
{
	HRESULT hr;
	ID3D11Device* device_3d=device->GetDevice3D();
	ID3D11Texture2D* texture_frame=NULL;

	ID3D11DeviceContext* context_3d=device->GetContext3D();
	bool result=false;
	if (c->data)
	{
		// eng: loading from memory
		// rus: загрузка из памяти
		result=device->LoadTextureFromMemory(c->data, c->length, true, &texture_frame);
	}
	else
	{
		// eng: loading from file
		// rus: загрузка из файла
		std::string texture_file=MP->GetPathToTexture();
		texture_file+=c->file;
		result=device->LoadTextureFromFile(texture_file.c_str(), true, &texture_frame);
	}

	assert(result);

	D3D11_TEXTURE2D_DESC info;
	texture_frame->GetDesc(&info);
	int wi_frame=info.Width;
	int he_frame=info.Height;

	if (wi_frame!=c->width || he_frame!=c->height)
	{
		// масштабируем текстуру
		D3D11_MAPPED_SUBRESOURCE maped_info;
		UINT subres=0;
		hr=context_3d->Map(texture_frame, subres,  D3D11_MAP_READ_WRITE, 0, &maped_info);
		unsigned int* pixels=(unsigned int*)maped_info.pData;
		int pitch=maped_info.RowPitch;

		float scale_x=((float)c->width)/wi_frame;
		float scale_y=((float)c->height)/he_frame;

		int pitch_to=pitch/4;
		int pitch_from=pitch_to;

		for (int i=0;i<c->width;i++)
		{
			for (int j=0;j<c->height;j++)
			{
				int i2=(int)(((float)i)/scale_x);
				int j2=(int)(((float)j)/scale_y);

				unsigned long color=pixels[j2*pitch_from+i2];
				pixels[j*pitch_to+i]=color;
			}
		}
		context_3d->Unmap(texture_frame, subres);
	}

	D3D11_BOX box;
	box.left=0;
	box.top=0;
	box.right=c->width;
	box.bottom=c->height;
	box.front=0;
	box.back=1;
	context_3d->CopySubresourceRegion(texture, 0, c->x, c->y, 0, texture_frame, 0, &box);
	
	texture_frame->Release();
	texture_frame=NULL;
}

// eng: Cleaning up of rectangle
// rus: Очистка прямоугольника
void MP_Atlas_WRAP::CleanRectangle(const MAGIC_CHANGE_ATLAS* c)
{
	ID3D11DeviceContext* context_3d=device->GetContext3D();

	D3D11_BOX box;
	box.left=c->x;
	box.top=c->y;
	box.right=c->x+c->width;
	box.bottom=c->y+c->height;
	box.front=0;
	box.back=1;

	int size=c->width*c->height;
	unsigned int* null_data=new unsigned int[size];
	ZeroMemory(null_data, size*4);
	
	context_3d->UpdateSubresource(texture, 0, &box, null_data, c->width*4, 0);

	delete []null_data;
	null_data=NULL;
}

// --------------------------------------------------------------------------------

MP_VERTEX_SHADER_WRAP::MP_VERTEX_SHADER_WRAP() : MP_VERTEX_SHADER()
{
	shader=NULL;
	shader_bytecode=NULL;
}

MP_VERTEX_SHADER_WRAP::~MP_VERTEX_SHADER_WRAP()
{
	if (shader)
	{
		shader->Release();
		shader=NULL;
	}

	if (shader_bytecode)
	{
		shader_bytecode->Release();
		shader_bytecode=NULL;
	}
}

void MP_VERTEX_SHADER_WRAP::Generate(MAGIC_MATERIAL* m, MP_Device* device)
{
	MP_VERTEX_SHADER::Generate(m, device);

	int i;
	MAGIC_VERTEX_FORMAT* format = &(m->format);
	bool is_color = (format->attributes & MAGIC_ATTRIBUTE_COLOR);
	int UVs = format->UVs;

	shader_code = "struct VS_OUTPUT\r\n";
	shader_code += "{\r\n";
	shader_code += "float4 Pos : SV_POSITION;\r\n";
	shader_code += "float4 Color : COLOR0;\r\n";
	for (i=0;i<UVs;i++)
	{
		shader_code += "float2 uv";
		shader_code += i + '0';
		shader_code += " : TEXCOORD";
		shader_code += i + '0';
		shader_code += ";\r\n";
	}
	shader_code += "};\r\n";
	#ifdef MAGIC_3D
	// Constant Buffer Variables
	shader_code += "cbuffer ConstantBuffer3d : register (b0)\r\n";
	shader_code += "{\r\n";
	shader_code += "matrix Smat;\r\n";
	shader_code += "};\r\n";
	#else
	// Constant Buffer Variables
	shader_code += "cbuffer ConstantBuffer2d : register (b0)\r\n";
	shader_code += "{\r\n";
	shader_code += "float2 ScreenScale;\r\n";
	shader_code += "};\r\n";
	#endif

	shader_code += "VS_OUTPUT main( float";
	#ifdef MAGIC_3D
	shader_code += "3";
	#else
	shader_code += "2";
	#endif
	shader_code += " Pos : POSITION";
	if (is_color)
		shader_code += ", float4 Color : COLOR";
	for (i=0;i<UVs;i++)
	{
		shader_code += ", float2 uv";
		shader_code += i + '0';
		shader_code += " : TEXCOORD";
		shader_code += i + '0';
	}
	shader_code += ")\r\n";
	shader_code += "{\r\n";
	shader_code += "VS_OUTPUT output=(VS_OUTPUT)0;\r\n";
	#ifdef MAGIC_3D
	shader_code += "output.Pos=mul(float4(Pos, 1),Smat);\r\n";
	#else
	shader_code += "output.Pos=float4((Pos*ScreenScale-1)*float2(1, -1),0,1);\r\n";
	#endif
	if (is_color)
		shader_code += "output.Color=Color;\r\n";
	else
		shader_code += "output.Color=float4(1,1,1,1);\r\n";

	for (i=0;i<UVs;i++)
	{
		shader_code += "output.uv";
		shader_code += i + '0';
		shader_code += "=uv";
		shader_code += i + '0';
		shader_code += ";\r\n";
	}
	shader_code += "return output;\r\n";
	shader_code += "};\r\n";

	HRESULT hr = D3DCompile(shader_code.c_str(), shader_code.length(), NULL, NULL, NULL, "main", "vs_4_0", D3DCOMPILE_ENABLE_STRICTNESS, 0, &shader_bytecode, NULL);
	assert(hr == S_OK);

	char* shader_data = (char*)shader_bytecode->GetBufferPointer();
	int shader_len = shader_bytecode->GetBufferSize();

	hr = ((MP_Device_WRAP*)device)->GetDevice3D()->CreateVertexShader(shader_data, shader_len, NULL, &shader);
	assert(hr == S_OK);
}

// -------------------------------------

MP_PIXEL_SHADER_WRAP::MP_PIXEL_SHADER_WRAP() : MP_PIXEL_SHADER()
{
	shader = NULL;
}

MP_PIXEL_SHADER_WRAP::~MP_PIXEL_SHADER_WRAP()
{
	if (shader)
	{
		shader->Release();
		shader = NULL;
	}
}

void MP_PIXEL_SHADER_WRAP::Generate(MAGIC_MATERIAL* m, MP_Device* device)
{
	MP_PIXEL_SHADER::Generate(m, device);

	int i;
	bool is_color=(m->format.attributes & MAGIC_ATTRIBUTE_COLOR);
	int textures = m->textures;

	for (int i = 0; i<textures; i++)
	{
		shader_code += "Texture2D txDiffuse";
		shader_code += i + '0';
		shader_code += ": register( t";
		shader_code += i + '0';
		shader_code += ");\r\n";
	}

	shader_code += "SamplerState sampler_linear : register( s0 );\r\n";
	shader_code += "struct VS_OUTPUT\r\n";
	shader_code += "{\r\n";
	shader_code+="float4 Pos : SV_POSITION;\r\n";
	shader_code+="float4 Color : COLOR0;\r\n";

	for (int i=0;i<textures;i++)
	{
		shader_code+="float2 uv";
		shader_code+=i+'0';
		shader_code += ": TEXCOORD";
		shader_code += i + '0';
		shader_code += ";\r\n";
	}
	shader_code += "};\r\n";

	shader_code += "float4 main(VS_OUTPUT input) : SV_TARGET\r\n";
	shader_code += "{\r\n";

	if (textures)
	{
		shader_code += "float4 color;\r\n";
		shader_code += "float4 arg1;\r\n";
		shader_code += "float4 arg2;\r\n";
		shader_code += "float4 colorTex;\r\n";
		shader_code += "float4 colorVarying=input.Color;\r\n";

		for (i = 0; i<textures; i++)
		{
			shader_code += "\r\n";
			//TEXTURE_STATES* s = &(texture_states[i]);
			MAGIC_TEXTURE_STATES* s = &(m->states[i]);

			if (s->argument_rgb1 == MAGIC_TEXARG_TEXTURE || s->argument_alpha1 == MAGIC_TEXARG_TEXTURE || (s->operation_rgb != MAGIC_TEXOP_ARGUMENT1 && s->argument_rgb2 == MAGIC_TEXARG_TEXTURE) || (s->operation_alpha != MAGIC_TEXOP_ARGUMENT1 && s->argument_alpha2 == MAGIC_TEXARG_TEXTURE))
			{
				shader_code += "colorTex = txDiffuse";
				shader_code += i + '0';
				shader_code+=".Sample( sampler_linear, input.uv";
				shader_code+=i+'0';
				shader_code+=" );\r\n";
			}

			if (s->argument_rgb1 == s->argument_alpha1)
			{
				switch (s->argument_rgb1)
				{
				case MAGIC_TEXARG_CURRENT:
					shader_code += "arg1 = color;\r\n";
					break;
				case MAGIC_TEXARG_DIFFUSE:
					shader_code += "arg1 = colorVarying;\r\n";
					break;
				default:
					shader_code += "arg1 = colorTex;\r\n";
					break;
				}
			}
			else
			{
				switch (s->argument_rgb1)
				{
				case MAGIC_TEXARG_CURRENT:
					shader_code += "arg1.xyz = color.xyz;\r\n";
					break;
				case MAGIC_TEXARG_DIFFUSE:
					shader_code += "arg1.xyz = colorVarying.xyz;\r\n";
					break;
				default:
					shader_code += "arg1.xyz = colorTex.xyz;\r\n";
					break;
				}
				switch (s->argument_alpha1)
				{
				case MAGIC_TEXARG_CURRENT:
					shader_code += "arg1.w = color.w;\r\n";
					break;
				case MAGIC_TEXARG_DIFFUSE:
					shader_code += "arg1.w = colorVarying.w;\r\n";
					break;
				default:
					shader_code += "arg1.w = colorTex.w;\r\n";
					break;
				}
			}

			if (s->argument_rgb2 == s->argument_alpha2 && s->operation_rgb != MAGIC_TEXOP_ARGUMENT1 && s->operation_alpha != MAGIC_TEXOP_ARGUMENT1)
			{
				switch (s->argument_rgb2)
				{
				case MAGIC_TEXARG_CURRENT:
					shader_code += "arg2 = color;\r\n";
					break;
				case MAGIC_TEXARG_DIFFUSE:
					shader_code += "arg2 = colorVarying;\r\n";
					break;
				default:
					shader_code += "arg2 = colorTex;\r\n";
					break;
				}
			}
			else
			{
				if (s->operation_rgb != MAGIC_TEXOP_ARGUMENT1)
				{
					switch (s->argument_rgb2)
					{
					case MAGIC_TEXARG_CURRENT:
						shader_code += "arg2.xyz = color.xyz;\r\n";
						break;
					case MAGIC_TEXARG_DIFFUSE:
						shader_code += "arg2.xyz = colorVarying.xyz;\r\n";
						break;
					default:
						shader_code += "arg2.xyz = colorTex.xyz;\r\n";
						break;
					}
				}
				if (s->operation_alpha != MAGIC_TEXOP_ARGUMENT1)
				{
					switch (s->argument_alpha2)
					{
					case MAGIC_TEXARG_CURRENT:
						shader_code += "arg2.w = color.w;\r\n";
						break;
					case MAGIC_TEXARG_DIFFUSE:
						shader_code += "arg2.w = colorVarying.w;\r\n";
						break;
					default:
						shader_code += "arg2.w = colorTex.w;\r\n";
						break;
					}
				}
			}

			if (s->operation_rgb == s->operation_alpha)
			{
				switch (s->operation_rgb)
				{
				case MAGIC_TEXOP_ARGUMENT1:
					shader_code += "color = arg1;\r\n";
					break;
				case MAGIC_TEXOP_ADD:
					shader_code += "color = arg1 + arg2;\r\n";
					break;
				case MAGIC_TEXOP_SUBTRACT:
					shader_code += "color = arg1 - arg2;\r\n";
					break;
				case MAGIC_TEXOP_MODULATE:
					shader_code += "color = arg1 * arg2;\r\n";
					break;
				case MAGIC_TEXOP_MODULATE2X:
					shader_code += "color = arg1 * arg2;\r\n";
					shader_code += "color = 2 * color;\r\n";
					break;
				default:
					shader_code += "color = arg1 * arg2;\r\n";
					shader_code += "color = 4 * color;\r\n";
					break;
				}
			}
			else
			{
				switch (s->operation_rgb)
				{
				case MAGIC_TEXOP_ARGUMENT1:
					shader_code += "color.xyz = arg1.xyz;\r\n";
					break;
				case MAGIC_TEXOP_ADD:
					shader_code += "color.xyz = arg1.xyz + arg2.xyz;\r\n";
					break;
				case MAGIC_TEXOP_SUBTRACT:
					shader_code += "color.xyz = arg1.xyz - arg2.xyz;\r\n";
					break;
				case MAGIC_TEXOP_MODULATE:
					shader_code += "color.xyz = arg1.xyz * arg2.xyz;\r\n";
					break;
				case MAGIC_TEXOP_MODULATE2X:
					shader_code += "color.xyz = arg1.xyz * arg2.xyz;\r\n";
					shader_code += "color.xyz = 2 * color.xyz;\r\n";
					break;
				default:
					shader_code += "color.xyz = arg1.xyz * arg2.xyz;\r\n";
					shader_code += "color.xyz = 4 * color.xyz;\r\n";
					break;
				}

				switch (s->operation_alpha)
				{
				case MAGIC_TEXOP_ARGUMENT1:
					shader_code += "color.w = arg1.w;\r\n";
					break;
				case MAGIC_TEXOP_ADD:
					shader_code += "color.w = arg1.w + arg2.w;\r\n";
					break;
				case MAGIC_TEXOP_SUBTRACT:
					shader_code += "color.w = arg1.w - arg2.w;\r\n";
					break;
				case MAGIC_TEXOP_MODULATE:
					shader_code += "color.w = arg1.w * arg2.w;\r\n";
					break;
				case MAGIC_TEXOP_MODULATE2X:
					shader_code += "color.w = arg1.w * arg2.w;\r\n";
					shader_code += "color.w = 2 * color.w;\r\n";
					break;
				default:
					shader_code += "color.w = arg1.w * arg2.w;\r\n";
					shader_code += "color.w = 4 * color.w;\r\n";
					break;
				}
			}
		}

		#ifdef SHADER_ALPHATEST_WRAP
		if (m->flags & MAGIC_MATERIAL_ALPHATEST)
		{
			// это альфа-тест
			shader_code += "clip(float4(color.x, color.y, color.z, color.w-1/255));\r\n";
		}
		#endif

		shader_code += "return color;\r\n";
	}
	else
	{
		shader_code += "return input.Color;\r\n";
	}
	shader_code += "}\r\n";


	ID3DBlob* pVSBlob = NULL;
	HRESULT hr = D3DCompile(shader_code.c_str(), shader_code.length(), NULL, NULL, NULL, "main", "ps_4_0", D3DCOMPILE_ENABLE_STRICTNESS, 0, &pVSBlob, NULL);
	assert(hr == S_OK);
	/*
	if (error)
	{
	const char* text=(const char*)error->GetBufferPointer();
	MessageBox(NULL, text, "Error", MB_OK | MB_ICONERROR);
	error->Release();
	error=NULL;
	}
	*/
	char* shader_data = (char*)pVSBlob->GetBufferPointer();
	int shader_len = pVSBlob->GetBufferSize();


	hr = ((MP_Device_WRAP*)device)->GetDevice3D()->CreatePixelShader(shader_data, shader_len, NULL, &(shader));

	pVSBlob->Release();
	pVSBlob = NULL;
}

// --------------------------------------------------------------------------------

MP_LAYOUT_WRAP::MP_LAYOUT_WRAP() : MP_LAYOUT()
{
	layout=NULL;
}

MP_LAYOUT_WRAP::~MP_LAYOUT_WRAP()
{
	if (layout)
	{
		layout->Release();
		layout=NULL;
	}
}

void MP_LAYOUT_WRAP::Generate(MAGIC_VERTEX_FORMAT* format, MP_Device* device)
{
	int offset=1;
	if (format->attributes & MAGIC_ATTRIBUTE_COLOR)
		offset++;

	int num=offset+format->UVs;
	D3D11_INPUT_ELEMENT_DESC* layout=new D3D11_INPUT_ELEMENT_DESC[num];
	int n=0;
	D3D11_INPUT_ELEMENT_DESC* l=&(layout[0]);
	l->SemanticName="POSITION";
    l->SemanticIndex=0;
	#ifdef MAGIC_3D
    l->Format=DXGI_FORMAT_R32G32B32_FLOAT;
	n+=12;
	#else
	l->Format=DXGI_FORMAT_R32G32_FLOAT;
	n+=8;
	#endif
    l->InputSlot=0;
    l->AlignedByteOffset=0;
	l->InputSlotClass=D3D11_INPUT_PER_VERTEX_DATA;
    l->InstanceDataStepRate=0;
	
	if (format->attributes & MAGIC_ATTRIBUTE_COLOR)
	{
		l=&(layout[1]);
		l->SemanticName="COLOR";
		l->SemanticIndex=0;
		l->Format=DXGI_FORMAT_R8G8B8A8_UNORM;
		l->InputSlot=0;
		l->AlignedByteOffset=n;
		l->InputSlotClass=D3D11_INPUT_PER_VERTEX_DATA;
		l->InstanceDataStepRate=0;
		n+=4;
	}

	for (int i=0;i<format->UVs;i++)
	{
		l=&(layout[offset+i]);
		l->SemanticName="TEXCOORD";
		l->SemanticIndex=i;
		l->Format=DXGI_FORMAT_R32G32_FLOAT;
		l->InputSlot=0;
		l->AlignedByteOffset=n;
		l->InputSlotClass=D3D11_INPUT_PER_VERTEX_DATA;
		l->InstanceDataStepRate=0;
		n+=8;
	}

	MP_VShaderList* list=device->GetVShaderList();
	MAGIC_MATERIAL temp_material;
	temp_material.format=*format;
	int vs_index=list->Select(&temp_material, device);

	MP_VERTEX_SHADER_WRAP* vs=(MP_VERTEX_SHADER_WRAP*)list->Get(vs_index);
	
	char* shader_data=(char*)vs->shader_bytecode->GetBufferPointer();
	int shader_len=vs->shader_bytecode->GetBufferSize();

	
	HRESULT hr=((MP_Device_WRAP*)device)->GetDevice3D()->CreateInputLayout(layout, num, shader_data, shader_len, &(this->layout));
	
	delete []layout;
	layout=NULL;
}

// -------------------------------------

MP_BUFFER_WRAP::MP_BUFFER_WRAP() : MP_BUFFER()
{
	buffer=NULL;
}

void MP_BUFFER_WRAP::Destroy()
{
	MP_BUFFER::Destroy();

	if (buffer)
	{
		buffer->Release();
		buffer=NULL;
	}
}

void* MP_BUFFER_WRAP::Map(int stride, MP_Device* device)
{
	ID3D11DeviceContext* context_3d=((MP_Device_WRAP*)device)->GetContext3D();
	D3D11_MAPPED_SUBRESOURCE mapped_buffer;
	HRESULT hr=context_3d->Map(buffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mapped_buffer);
	assert(hr==S_OK);
	return mapped_buffer.pData;
}

void MP_BUFFER_WRAP::Unmap(MP_Device* device)
{
	ID3D11DeviceContext* context_3d=((MP_Device_WRAP*)device)->GetContext3D();
	context_3d->Unmap(buffer, 0);
}

// -------------------------------------

MP_VERTEX_BUFFER_WRAP::MP_VERTEX_BUFFER_WRAP() : MP_BUFFER_WRAP()
{
}

void MP_VERTEX_BUFFER_WRAP::Create(int new_length, MP_Device* device)
{
	MP_BUFFER_WRAP::Create(new_length, device);

	D3D11_BUFFER_DESC bd;
	ZeroMemory( &bd, sizeof(bd) );
	bd.Usage=D3D11_USAGE_DYNAMIC;
	bd.ByteWidth=new_length;
	bd.BindFlags=D3D11_BIND_VERTEX_BUFFER;
	bd.CPUAccessFlags=D3D11_CPU_ACCESS_WRITE;

	ID3D11Device* device_3d=((MP_Device_WRAP*)device)->GetDevice3D();
	HRESULT hr=device_3d->CreateBuffer(&bd, NULL, &buffer);
	assert(hr==S_OK);
}

void MP_VERTEX_BUFFER_WRAP::Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device)
{
	ID3D11DeviceContext* context_3d=((MP_Device_WRAP*)device)->GetContext3D();
	UINT offset=0;
	UINT s=stride;
	context_3d->IASetVertexBuffers( 0, 1, &buffer, &s, &offset );
}

// -------------------------------------

MP_INDEX_BUFFER_WRAP::MP_INDEX_BUFFER_WRAP() : MP_BUFFER_WRAP()
{
}

void MP_INDEX_BUFFER_WRAP::Create(int new_length, MP_Device* device)
{
	MP_BUFFER_WRAP::Create(new_length, device);

	D3D11_BUFFER_DESC bd;
	ZeroMemory(&bd, sizeof(bd) );
	bd.Usage=D3D11_USAGE_DYNAMIC;
	bd.ByteWidth=new_length;
	bd.BindFlags=D3D11_BIND_INDEX_BUFFER;
	bd.CPUAccessFlags=D3D11_CPU_ACCESS_WRITE;

	ID3D11Device* device_3d=((MP_Device_WRAP*)device)->GetDevice3D();
	HRESULT hr=device_3d->CreateBuffer(&bd, NULL, &buffer);
	assert(hr==S_OK);
}

void MP_INDEX_BUFFER_WRAP::Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device)
{
	#ifdef INDEX_BUFFER_32_WRAP
	DXGI_FORMAT index_size=DXGI_FORMAT_R32_UINT;
	#else
	DXGI_FORMAT index_size=DXGI_FORMAT_R16_UINT;
	#endif
	
	ID3D11DeviceContext* context_3d=((MP_Device_WRAP*)device)->GetContext3D();
	context_3d->IASetIndexBuffer(buffer, index_size, 0 );
}

// --------------------------------------------------------------------------------

// eng: Class controlling drawing
// rus: Класс, который управляет рисованием
#ifdef EXT_DEVICE
MP_Device_WRAP::MP_Device_WRAP(int width, int height, ID3D11Device* device_3d, ID3D11DeviceContext* context_3d) : MP_Device(width, height)
{
	this->device_3d=device_3d;
	this->context_3d=context_3d;
#else
MP_Device_WRAP::MP_Device_WRAP(int width, int height, HWND hwnd) : MP_Device(width, height)
{
	this->hwnd=hwnd;

	device_3d=NULL;
	context_3d=NULL;
	swap_chain=NULL;
	target_view=NULL;
#endif
	constant_buffer=NULL;
	rasterizer_state=NULL;
	blend_state_normal=blend_state_add=blend_state_opacity=NULL;

	for (int i=0;i<2;i++)
		for (int j=0;j<2;j++)
			depth_states[i][j]=NULL;

	zbuffer_enabled=false;
	zwrite_enabled=false;
	z_changed=false;

	for (int i=0;i<4;i++)
		for (int j=0;j<4;j++)
			sampler_states[i][j]=NULL;
	

	k_changed_address=0;
	for (int i=0;i<MAX_SAMPLERS;i++)
	{
		TEXTURE_STATES* tex_states=&(texture_states[i]);
		tex_states->sampler=i;
	}
}

// eng: Creating
// rus: Создание
bool MP_Device_WRAP::Create()
{
	MP_Device::Create();

	bool result=false;

	HRESULT hr;

	#ifndef EXT_DEVICE
	RECT rc;
    GetClientRect(hwnd, &rc );
    window_width=rc.right-rc.left;	// получаем ширину
    window_height=rc.bottom-rc.top;	// и высоту окна

	#ifdef _DEBUG
	UINT createDeviceFlags=D3D11_CREATE_DEVICE_DEBUG;
	#else
    UINT createDeviceFlags=0;
	#endif

	D3D_DRIVER_TYPE g_driverType = D3D_DRIVER_TYPE_NULL;
	D3D_FEATURE_LEVEL g_featureLevel = D3D_FEATURE_LEVEL_11_0;

    D3D_DRIVER_TYPE driverTypes[] =
    {
        D3D_DRIVER_TYPE_HARDWARE,
        D3D_DRIVER_TYPE_WARP,
        D3D_DRIVER_TYPE_REFERENCE,
    };
    UINT numDriverTypes = ARRAYSIZE( driverTypes );

    // Тут мы создаем список поддерживаемых версий DirectX
    D3D_FEATURE_LEVEL featureLevels[] =
    {
        D3D_FEATURE_LEVEL_11_0,
        D3D_FEATURE_LEVEL_10_1,
        D3D_FEATURE_LEVEL_10_0,
    };
	UINT numFeatureLevels = ARRAYSIZE( featureLevels );

	// Сейчас мы создадим устройства DirectX. Для начала заполним структуру,
	// которая описывает свойства переднего буфера и привязывает его к нашему окну.
    DXGI_SWAP_CHAIN_DESC sd;		// Структура, описывающая цепь связи (Swap Chain)
    ZeroMemory(&sd, sizeof(sd));	// очищаем ее
    sd.BufferCount=1;					// у нас один задний буфер
    sd.BufferDesc.Width=window_width;		// ширина буфера
    sd.BufferDesc.Height=window_height;		// высота буфера
    sd.BufferDesc.Format=DXGI_FORMAT_R8G8B8A8_UNORM;	// формат пикселя в буфере
    sd.BufferDesc.RefreshRate.Numerator=75;			// частота обновления экрана
    sd.BufferDesc.RefreshRate.Denominator=1;
    sd.BufferUsage=DXGI_USAGE_RENDER_TARGET_OUTPUT;	// назначение буфера - задний буфер
    sd.OutputWindow=hwnd;							// привязываем к нашему окну
    sd.SampleDesc.Count=1;
    sd.SampleDesc.Quality=0;
    sd.Windowed = TRUE;						// не полноэкранный режим

    for( UINT driverTypeIndex = 0; driverTypeIndex < numDriverTypes; driverTypeIndex++ )
    {
        g_driverType=driverTypes[driverTypeIndex];
        hr=D3D11CreateDeviceAndSwapChain(NULL, g_driverType, NULL, createDeviceFlags, featureLevels, numFeatureLevels,
                                            D3D11_SDK_VERSION, &sd, &swap_chain, &device_3d, &g_featureLevel, &context_3d);
        if (SUCCEEDED(hr)) // Если устройства созданы успешно, то выходим из цикла
			break;
    }

    // Теперь создаем задний буфер. Обратите внимание, в SDK
    // RenderTargetOutput - это передний буфер, а RenderTargetView - задний.
    ID3D11Texture2D* pBackBuffer=NULL;
    hr=swap_chain->GetBuffer( 0, __uuidof(ID3D11Texture2D), (LPVOID*)&pBackBuffer);
	assert(hr==S_OK);
    
    // Я уже упоминал, что интерфейс g_pd3dDevice будет
    // использоваться для создания остальных объектов
    hr=device_3d->CreateRenderTargetView( pBackBuffer, NULL, &target_view);
    pBackBuffer->Release();
    assert(hr==S_OK);

    // Подключаем объект заднего буфера к контексту устройства
    context_3d->OMSetRenderTargets( 1, &target_view, NULL );
	#endif

	D3D11_BUFFER_DESC bd;
	ZeroMemory(&bd, sizeof(bd));
	bd.Usage=D3D11_USAGE_DEFAULT;
	bd.ByteWidth=sizeof(CONSTANT_BUFFER);
	bd.BindFlags=D3D11_BIND_CONSTANT_BUFFER;
	bd.CPUAccessFlags=0;
    device_3d->CreateBuffer( &bd, NULL, &constant_buffer);

	// Растеризатор
	D3D11_RASTERIZER_DESC rd;
	rd.FillMode=D3D11_FILL_SOLID;
	rd.CullMode=D3D11_CULL_NONE;
	rd.FrontCounterClockwise=TRUE;
	rd.DepthBias=0;
	rd.SlopeScaledDepthBias=0.0f;
	rd.DepthBiasClamp=0.0f;
	rd.DepthClipEnable=TRUE;
	rd.ScissorEnable=FALSE;
	rd.MultisampleEnable=FALSE;
	rd.AntialiasedLineEnable=FALSE;

	device_3d->CreateRasterizerState(&rd, &rasterizer_state);

	D3D11_BLEND_DESC BlendStateDesc;
	ZeroMemory(&BlendStateDesc, sizeof(D3D11_BLEND_DESC));
	BlendStateDesc.AlphaToCoverageEnable=FALSE;
	BlendStateDesc.IndependentBlendEnable=FALSE;
	D3D11_RENDER_TARGET_BLEND_DESC* brt=&(BlendStateDesc.RenderTarget[0]);
	brt->BlendEnable=TRUE; // включаем смешивание для RenderTarget#0
	brt->SrcBlend=D3D11_BLEND_SRC_ALPHA; // Для операции смешивания цвет источника будет умножен на значение альфа
	brt->DestBlend=D3D11_BLEND_INV_SRC_ALPHA; // Для операции смешивания цвет уже отрисованного пикселя будет умножен на значение 1 - альфа
	brt->BlendOpAlpha=D3D11_BLEND_OP_ADD;
	brt->DestBlendAlpha=D3D11_BLEND_INV_SRC_ALPHA;
	brt->RenderTargetWriteMask=D3D11_COLOR_WRITE_ENABLE_ALL; // Маска записи для RT#0. Писать все.
	brt->BlendOp=D3D11_BLEND_OP_ADD;
	brt->SrcBlendAlpha=D3D11_BLEND_ONE;
	brt->DestBlendAlpha=D3D11_BLEND_INV_SRC_ALPHA;
	brt->BlendOpAlpha=D3D11_BLEND_OP_ADD;
	brt->RenderTargetWriteMask=D3D11_COLOR_WRITE_ENABLE_ALL;

	hr=device_3d->CreateBlendState(&BlendStateDesc, &blend_state_normal);
	assert(hr==S_OK);

	brt->DestBlend = D3D11_BLEND_ONE;
	hr=device_3d->CreateBlendState(&BlendStateDesc, &blend_state_add);
	assert(hr==S_OK);

	brt->BlendEnable=FALSE;
	hr=device_3d->CreateBlendState(&BlendStateDesc, &blend_state_opacity);
	assert(hr==S_OK);



	D3D11_DEPTH_STENCIL_DESC DepthDesc;
	
	// Initialize the description of the stencil state.
	ZeroMemory(&DepthDesc, sizeof(DepthDesc));

	// Set up the description of the stencil state.
	DepthDesc.DepthEnable = false;
	DepthDesc.DepthWriteMask = D3D11_DEPTH_WRITE_MASK_ZERO;
	DepthDesc.DepthFunc = D3D11_COMPARISON_LESS_EQUAL;

	DepthDesc.StencilEnable = false;
	DepthDesc.StencilReadMask = 0xFF;
	DepthDesc.StencilWriteMask = 0xFF;

	// Stencil operations if pixel is front-facing.
	DepthDesc.FrontFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
	DepthDesc.FrontFace.StencilDepthFailOp = D3D11_STENCIL_OP_INCR;
	DepthDesc.FrontFace.StencilPassOp = D3D11_STENCIL_OP_KEEP;
	DepthDesc.FrontFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

	// Stencil operations if pixel is back-facing.
	DepthDesc.BackFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
	DepthDesc.BackFace.StencilDepthFailOp = D3D11_STENCIL_OP_DECR;
	DepthDesc.BackFace.StencilPassOp = D3D11_STENCIL_OP_KEEP;
	DepthDesc.BackFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

	hr=device_3d->CreateDepthStencilState(&DepthDesc, &(depth_states[0][0]));

	DepthDesc.DepthEnable = true;
	hr=device_3d->CreateDepthStencilState(&DepthDesc, &(depth_states[1][0]));

	DepthDesc.DepthWriteMask = D3D11_DEPTH_WRITE_MASK_ALL;
	hr=device_3d->CreateDepthStencilState(&DepthDesc, &(depth_states[1][1]));

	DepthDesc.DepthEnable = false;
	hr=device_3d->CreateDepthStencilState(&DepthDesc, &(depth_states[0][1]));
	assert(hr==S_OK);
	
	// Создание объекта состояний семплера
	D3D11_SAMPLER_DESC sampDesc;
	ZeroMemory( &sampDesc, sizeof(sampDesc) );
	sampDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	sampDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
	sampDesc.ComparisonFunc = D3D11_COMPARISON_NEVER;
	sampDesc.MinLOD = 0;
	sampDesc.MaxLOD = D3D11_FLOAT32_MAX;
	
	D3D11_TEXTURE_ADDRESS_MODE address_mode[]={D3D11_TEXTURE_ADDRESS_WRAP, D3D11_TEXTURE_ADDRESS_MIRROR, D3D11_TEXTURE_ADDRESS_CLAMP, D3D11_TEXTURE_ADDRESS_BORDER};
	for (int i=0; i<4; i++)
	{
		sampDesc.AddressU=address_mode[i];
		for (int j=0; j<4; j++)
		{
			sampDesc.AddressV=address_mode[j];
			hr=device_3d->CreateSamplerState(&sampDesc, &(sampler_states[i][j]));
			assert(hr==S_OK);
		}
	}


	#ifndef EXT_DEVICE
	D3D11_VIEWPORT view_port;
    view_port.Width=(float)window_width;
    view_port.Height=(float)window_height;
    view_port.MinDepth=0.0f;
    view_port.MaxDepth=1.0f;
    view_port.TopLeftX=0;
    view_port.TopLeftY=0;
    // Подключаем вьюпорт к контексту устройства
    context_3d->RSSetViewports(1, &view_port);
	#endif

	result=true;

	return result;
}

#ifdef MAGIC_3D
// eng: Setting of scene
// rus: Настройка сцены
void MP_Device_WRAP::SetScene3d(bool camera_mode)
{
	MP_Device::SetScene3d(camera_mode);
	// eng: projection matrix
	// rus: проектная матрица

	float znear=1.f;
	float zfar=1000000.f;
	if (camera_mode)
		projection_matrix.MatrixPerspectiveFovLH((float)(MAGIC_PI * 0.15f), ((float)window_width)/((float)window_height), znear, zfar);
	else
		projection_matrix.MatrixOrthoLH((float)window_width, (float)window_height, -10000.f, 10000.f);
	
	MP_POSITION eye(0.f,300.f,-840.f);
	MP_POSITION at(0.f,0.f,0.f);
	MP_POSITION up(0.f,1.f,0.f);

	SetCamera(eye, at, up);
}

// eng: Camera position
// rus: Положение камеры
void MP_Device_WRAP::SetCamera(MP_POSITION& eye, MP_POSITION& at, MP_POSITION& up)
{
	MP_Device::SetCamera(eye, at, up);

	// eng: view matrix
	// rus: матрица вида
	view_matrix.MatrixLookAtLH(&eye, &at, &up);
}
#endif

// eng: Beginning of scene drawing
// rus: Начало отрисовки сцены
void MP_Device_WRAP::BeginScene()
{
	MP_Device::BeginScene();

	#ifndef EXT_DEVICE
	float ClearColor[4] = { 0.0f, 0.0f, 0.0f, 1.0f }; // красный, зеленый, синий, альфа-канал
	context_3d->ClearRenderTargetView(target_view, ClearColor);
	#endif

	context_3d->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

	// Установка констант шейдера
	CONSTANT_BUFFER cb;
	#ifdef MAGIC_3D
	cb.Smat.MatrixMul(&view_matrix, &projection_matrix);
	cb.Smat.Transposing();
	#else
	cb.screen_scale.x=2.f/(float)window_width;
	cb.screen_scale.y=2.f/(float)window_height;
	#endif
	context_3d->UpdateSubresource(constant_buffer, 0, NULL, &cb, 0, 0 );

	UINT start_slot=0;
	
	context_3d->VSSetConstantBuffers(start_slot, 1, &constant_buffer);

	context_3d->RSSetState(rasterizer_state);
}

// eng: End of scene drawing
// rus: Конец отрисовки сцены
void MP_Device_WRAP::EndScene()
{
	#ifndef EXT_DEVICE
	swap_chain->Present( 0, 0 );
	#endif
}

// eng: Indicates that device is lost
// rus: Проверка на потерю устройства рисования
bool MP_Device_WRAP::IsLost()
{
	return false;
}

void MP_Device_WRAP::Destroy()
{
	MP_Device::Destroy();

	if (context_3d)
	{
		context_3d->ClearState();
	}

	if (constant_buffer)
	{
		constant_buffer->Release();
		constant_buffer=NULL;
	}

	if (blend_state_normal)
	{
		blend_state_normal->Release();
		blend_state_normal=NULL;
	}

	if (blend_state_add)
	{
		blend_state_add->Release();
		blend_state_add=NULL;
	}

	if (blend_state_opacity)
	{
		blend_state_opacity->Release();
		blend_state_opacity=NULL;
	}

	for (int i=0;i<2;i++)
	{
		for (int j=0;j<2;j++)
		{
			if (depth_states[i][j])
			{
				depth_states[i][j]->Release();
				depth_states[i][j]=NULL;
			}
		}
	}

	if (rasterizer_state)
	{
		rasterizer_state->Release();
		rasterizer_state=NULL;
	}

	for (int i=0;i<4;i++)
	{
		for (int j=0;j<4;j++)
		{
			if (sampler_states[i][j])
			{
				sampler_states[i][j]->Release();
				sampler_states[i][j]=NULL;
			}
		}
	}

	#ifndef EXT_DEVICE
    if (target_view)
	{
		target_view->Release();
		target_view=NULL;
	}

    if (swap_chain)
	{
		swap_chain->Release();
		swap_chain=NULL;
	}

    if (context_3d)
	{
		context_3d->Release();
		context_3d=NULL;
	}

	if (device_3d)
	{
		#ifdef _DEBUG
		ID3D11Debug* device_3d_debug=NULL;
		device_3d->QueryInterface(IID_ID3D11Debug, (void**) &device_3d_debug); 
		device_3d->Release();
		device_3d=NULL;
		device_3d_debug->ReportLiveDeviceObjects(D3D11_RLDO_DETAIL);
		device_3d_debug->Release();
		device_3d_debug=NULL;
		#else
		device_3d->Release();
		device_3d=NULL;
		#endif
	}

	#endif
}

bool MP_Device_WRAP::LoadTextureFromMemory(const char* data, int len, bool system_memory, ID3D11Texture2D** texture)
{
	bool result=false;
	D3D11_USAGE usage=D3D11_USAGE_DEFAULT;
	unsigned int bindFlags=D3D11_BIND_SHADER_RESOURCE;
	unsigned int cpuAccessFlags=0;
	if (system_memory)
	{
		usage=D3D11_USAGE_STAGING;
		bindFlags=0;
		cpuAccessFlags=D3D11_CPU_ACCESS_WRITE | D3D11_CPU_ACCESS_READ;
	}

	bool dds=(len>=3 && data[0]=='D' && data[1]=='D' && data[2]=='S');
	if (dds)
	{
		if (CreateDDSTextureFromMemoryEx(device_3d, (const uint8_t*)data, len, 0, usage, bindFlags, cpuAccessFlags, 0, false, (ID3D11Resource**)texture, NULL, NULL)==S_OK)
			result=true;
	}
	else
	{
		int n;
		int width, height;
		unsigned char* pixels=stbi_load_from_memory((const stbi_uc*)data, len, &width, &height, &n, 4);
		if (!pixels)
		{
			const char* error=stbi_failure_reason();
			//MessageBox(NULL, error, "Magic Particles", MB_OK | MB_ICONERROR);
			assert(false);
		}
		else
		{
			D3D11_TEXTURE2D_DESC desc;
			ZeroMemory(&desc, sizeof(desc));
			desc.Width=width;
			desc.Height=height;
			desc.MipLevels=1;
			desc.ArraySize=1;
			desc.Format=DXGI_FORMAT_R8G8B8A8_UNORM;
			desc.SampleDesc.Count=1;
			desc.SampleDesc.Quality=0;
			desc.Usage=usage;
			desc.BindFlags=bindFlags;
			desc.CPUAccessFlags=cpuAccessFlags;
			desc.MiscFlags=0;

			D3D11_SUBRESOURCE_DATA init_data;
			init_data.pSysMem=pixels;
			init_data.SysMemPitch=width*4;
			init_data.SysMemSlicePitch=init_data.SysMemPitch*height;
			HRESULT hr=device_3d->CreateTexture2D(&desc, &init_data, texture);
			assert(hr==S_OK);

			stbi_image_free(pixels);
			result=true;
		}
	}
	return result;
}

bool MP_Device_WRAP::LoadTextureFromFile(const char* texture_file, bool system_memory, ID3D11Texture2D** texture)
{
	std::string mode="rb";
	const wchar_t* pmode=MP->utf8_to_wchar(mode.c_str());
	wchar_t wmode[3];
	wmode[0]=pmode[0];
	wmode[1]=pmode[1];
	wmode[2]=0;

	const wchar_t* wfile=MP->utf8_to_wchar(texture_file);

	FILE* file=_wfopen(wfile, wmode);

	fseek(file,0,SEEK_END);
	long len=ftell(file);
	fseek(file,0,SEEK_SET);

	char* data=new char[len];

	fread(data,1,len,file);
	fclose(file);

	bool result=LoadTextureFromMemory(data, len, system_memory, texture);
	delete []data;
	data=NULL;

	return result;
}

// rus: Установка формата вершин
void MP_Device_WRAP::SetVertexFormat(MAGIC_RENDERING_START* start)
{
	MAGIC_VERTEX_FORMAT* format=&(start->format);
	int index=layout_list.Select(format, this);
	MP_LAYOUT_WRAP* layout=(MP_LAYOUT_WRAP*)layout_list.Get(index);
	context_3d->IASetInputLayout(layout->layout);
}

void MP_Device_WRAP::DrawVertices(int starting_index, int indexes_count, int max_vertices)
{
	context_3d->DrawIndexed(indexes_count, starting_index, 0);
}

void MP_Device_WRAP::SetRenderMaterial(int material)
{
	MP_MATERIAL* m = m_material[material];
	MP_TECHNIQUE* tech = (MP_TECHNIQUE*)technique_list.Get(m->technique_index);

	MP_VERTEX_SHADER_WRAP* vs=(MP_VERTEX_SHADER_WRAP*)vshader_list.Get(tech->vertex_shader_index);
	MP_PIXEL_SHADER_WRAP* ps=(MP_PIXEL_SHADER_WRAP*)pshader_list.Get(tech->pixel_shader_index);

	context_3d->VSSetShader(vs->shader, NULL, 0);
	context_3d->PSSetShader(ps->shader, NULL, 0);

	for (int i = 0; i<k_changed_address; i++)
	{
		int index = m_changed_address[i];
		TEXTURE_STATES* tex_states = &(texture_states[index]);
		tex_states->changed_address = false;
		context_3d->PSSetSamplers(tex_states->sampler, 1, &(sampler_states[tex_states->u_address][tex_states->v_address]));
	}
	k_changed_address = 0;

	if (z_changed)
	{
		context_3d->OMSetDepthStencilState(depth_states[zbuffer_enabled][zwrite_enabled], 1);
		z_changed = false;
	}
}

// -------------------------------------------------------------------------------------------

// eng: Creating of vertex shader
// rus: Создание вершинного шейдера
MP_ELEMENT* MP_Device_WRAP::NewVertexShader()
{
	return new MP_VERTEX_SHADER_WRAP;
}

// eng: Creating of pixel shader
// rus: Создание пиксельного шейдера
MP_ELEMENT* MP_Device_WRAP::NewPixelShader()
{
	return new MP_PIXEL_SHADER_WRAP;
}

// eng: Creating of technique
// rus: Создание техники
MP_ELEMENT* MP_Device_WRAP::NewTechnique()
{
	return new MP_TECHNIQUE;
}

// eng: Creating of layout
// rus: Создание декларации вершин
MP_LAYOUT* MP_Device_WRAP::NewLayout()
{
	return new MP_LAYOUT_WRAP;
}

// eng: Creating of vertex buffer
// rus: Создание вершинного буфера
MP_BUFFER* MP_Device_WRAP::NewVertexBuffer()
{
	return new MP_VERTEX_BUFFER_WRAP();
}

// eng: Creating of index buffer
// rus: Создание индексного буфера
MP_BUFFER* MP_Device_WRAP::NewIndexBuffer()
{
	return new MP_INDEX_BUFFER_WRAP();
}

// -------------------------------------------------------------------------------------------

// Обычное смешивание
void SetBlendingNormal(MP_Device_WRAP* device)
{
	ID3D11DeviceContext* context_3d=device->GetContext3D();
	float blend_factor_new[4]={1,1,1,1};
	context_3d->OMSetBlendState(device->blend_state_normal, blend_factor_new, 0xffffffff);
}

// Аддитивное смешивание
void SetBlendingAdd(MP_Device_WRAP* device)
{
	ID3D11DeviceContext* context_3d=device->GetContext3D();
	float blend_factor_new[4]={1,1,1,1};
	context_3d->OMSetBlendState(device->blend_state_add, blend_factor_new, 0xffffffff);
}

// Непрозрачность
void SetBlendingOpacity(MP_Device_WRAP* device)
{
	ID3D11DeviceContext* context_3d=device->GetContext3D();
	float blend_factor_new[4]={1,1,1,1};
	context_3d->OMSetBlendState(device->blend_state_opacity, blend_factor_new, 0xffffffff);
}

// массив с функциями, которые служат для установки блендинга
void (*render_blending[])(MP_Device_WRAP* device)={SetBlendingNormal, SetBlendingAdd, SetBlendingOpacity};

// Установка текущего смешивания
void SetRenderBlending(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	render_blending[s->value](device);
}

// Установка текущей текстуры
void SetRenderTexture(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	ID3D11DeviceContext* context_3d=device->GetContext3D();
	MP_Atlas_WRAP* atlas=(MP_Atlas_WRAP*)device->MP->GetAtlas(s->value);
	ID3D11ShaderResourceView* resource_view=atlas->GetResourceView();
	context_3d->PSSetShaderResources(s->index, 1, &resource_view);
}

// ----------------------------------------------------------------

// Установка текущей адресации по U
void SetRenderAddressU(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	TEXTURE_STATES* tex_states=&(device->texture_states[s->index]);
	tex_states->u_address=s->value;
	if (!tex_states->changed_address)
	{
		tex_states->changed_address=true;
		device->m_changed_address[device->k_changed_address]=s->index;
		device->k_changed_address++;
	}
}

// Установка текущей адресации по V
void SetRenderAddressV(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	TEXTURE_STATES* tex_states=&(device->texture_states[s->index]);
	tex_states->v_address=s->value;
	if (!tex_states->changed_address)
	{
		tex_states->changed_address=true;
		device->m_changed_address[device->k_changed_address]=s->index;
		device->k_changed_address++;
	}
}

// ----------------------------------------------------------------

// Включение/отключение проверки на Z-буфера
void SetRenderZEnable(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	device->zbuffer_enabled=s->value;
	device->z_changed=true;
}

// ----------------------------------------------------------------

// Настройка Z-буфера на перезапись
void SetRenderZWrite(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	device->zwrite_enabled=s->value;
	device->z_changed=true;
}

// ----------------------------------------------------------------

// Массив с функциями, которые служат для установки состояния видеокарты
void (*render_states[])(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)={SetRenderBlending, NULL, SetRenderTexture, SetRenderAddressU, SetRenderAddressV, NULL, NULL, NULL, NULL, NULL, NULL, SetRenderZEnable, SetRenderZWrite, NULL, NULL};

void MP_Device_WRAP::SetRenderState(MAGIC_RENDER_STATE* state)
{
	render_states[state->state](this, state);
}
