﻿#include "pch.h"
#include "dx2dMain.h"
#include "Common\DirectXHelper.h"
#include "platform_win8.h"

using namespace dx2d;

using namespace Windows::Foundation;
using namespace Windows::System::Threading;
using namespace Concurrency;

// Загружает и инициализирует ресурсы приложения во время его загрузки.
dx2dMain::dx2dMain(const std::shared_ptr<DX::DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	// Регистрация для получения уведомлений о том, что устройство потеряно или создано заново
	m_deviceResources->RegisterDeviceNotify(this);

	// TODO: Измените настройки таймера, если требуется нечто, отличное от режима по умолчанию с переменным шагом по времени.
	// например, для логики обновления с фиксированным временным шагом 60 кадров/с вызовите:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/


	Windows::Foundation::Size size = m_deviceResources->GetOutputSize();

	int client_wi = (int)size.Width;
	int client_he = (int)size.Height;

	MP_Manager& MP = MP_Manager::GetInstance();

	#ifdef MAGIC_3D
	MAGIC_AXIS_ENUM axis = MAGIC_pXpYpZ;
	#else
	MAGIC_AXIS_ENUM axis = MAGIC_pXpY;
	#endif
	MP_Platform* platform = new MP_Platform_WIN8;

	bool filters[MAGIC_RENDER_STATE__MAX];
	for (int i = 0; i<MAGIC_RENDER_STATE__MAX; i++)
		filters[i] = false;
	filters[MAGIC_RENDER_STATE_BLENDING] = true;
	filters[MAGIC_RENDER_STATE_TEXTURE] = true;
	filters[MAGIC_RENDER_STATE_ADDRESS_U] = true;
	filters[MAGIC_RENDER_STATE_ADDRESS_V] = true;
	filters[MAGIC_RENDER_STATE_ZENABLE] = true;
	filters[MAGIC_RENDER_STATE_ZWRITE] = true;

	MP.Initialization(filters, true, axis, platform, MAGIC_INTERPOLATION_ENABLE, MAGIC_CHANGE_EMITTER_DEFAULT, 1024, 1024, 1, 1.f, 0.1f, true);

	#if (WINAPI_FAMILY==WINAPI_FAMILY_APP)
	if (client_wi<client_he)
	{
		int v = client_he;
		client_he = client_wi;
		client_wi = v;
	}
	#endif

	magic_device = new MP_Device_WRAP(client_wi, client_he, m_deviceResources->GetD3DDevice(), m_deviceResources->GetD3DDeviceContext());
	magic_device->Create();

	// eng: find of all ptc-files in folder
	// rus: поиск всех ptc-файлов в папке
	MP.LoadAllEmitters();

	MP.RefreshAtlas();

	MP.CloseFiles();

	MP.Stop();

	cur=MP.GetFirstEmitter();

	cur->SetState(MAGIC_STATE_UPDATE);

	#ifdef MAGIC_3D
	// eng: set view matrix
	// rus: устанавливаем матрицу вида
	magic_device->SetScene3d();
	#else
	// eng: locate emitters the same as editor
	// rus: расставляем эмиттеры также, как они стояли в редакторе
	MP_Emitter* emitter=MP.GetFirstEmitter();
	while(emitter)
	{
		Magic_CorrectEmitterPosition(emitter->GetEmitter(), client_wi, client_he);
		emitter=emitter->GetNext();
	}
	#endif
}

dx2dMain::~dx2dMain()
{
	// Отмена регистрации уведомлений устройства
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Обновляет состояние приложения при изменении размера окна (например, при изменении ориентации устройства)
void dx2dMain::CreateWindowSizeDependentResources() 
{
	
	
}

// Обновляет состояние приложения один раз за кадр.
void dx2dMain::Update() 
{
	// Обновите объекты сцены.
	m_timer.Tick([&]()
	{
		// TODO: Замените это функциями обновления содержимого вашего приложения.
		//m_sceneRenderer->Update(m_timer);
		//m_fpsTextRenderer->Update(m_timer);
		MP_Manager& MP = MP_Manager::GetInstance();
		MP.UpdateByTimer();
	});
}

// Производит отрисовку текущего кадра в соответствии с текущим состоянием приложения.
// Возвращает значение true, если кадр отрисован и готов к отображению.
bool dx2dMain::Render() 
{
	// Не пытайтесь выполнять какую-либо отрисовку до первого обновления.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

	auto context = m_deviceResources->GetD3DDeviceContext();

	// Выполните сброс окна просмотра для нацеливания на весь экран.
	auto viewport = m_deviceResources->GetScreenViewport();
	context->RSSetViewports(1, &viewport);

	// Выполните сброс экрана в качестве целевого объекта прорисовки.
	ID3D11RenderTargetView *const targets[1] = { m_deviceResources->GetBackBufferRenderTargetView() };
	context->OMSetRenderTargets(1, targets, m_deviceResources->GetDepthStencilView());

	// Очистите задний буфер и представление трафарета глубины.
	context->ClearRenderTargetView(m_deviceResources->GetBackBufferRenderTargetView(), DirectX::Colors::Black);
	context->ClearDepthStencilView(m_deviceResources->GetDepthStencilView(), D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL, 1.0f, 0);

	// Выполните отрисовку объектов сцены.
	// TODO: Замените это функциями отрисовки содержимого вашего приложения.
	//m_sceneRenderer->Render();
	//m_fpsTextRenderer->Render();


	MP_Manager& MP = MP_Manager::GetInstance();

	MP.Render();

	if (cur->GetState()==MAGIC_STATE_STOP)
	{
		cur=cur->GetNext();
		if (!cur)
			cur=MP.GetFirstEmitter();

		cur->SetState(MAGIC_STATE_UPDATE);
	}

	return true;
}

// Уведомляет визуализаторы о том, что ресурсы устройства необходимо освободить.
void dx2dMain::OnDeviceLost()
{
	
}

// Уведомляет визуализаторы о том, что ресурсы устройства можно создать заново.
void dx2dMain::OnDeviceRestored()
{

	CreateWindowSizeDependentResources();
}
